#2FitModels.R

#Code to fit the statistical models in this published paper:
#New methods for quantifying the effects of catchment spatial patterns on aquatic responses
#by Donald E. Weller, Matthew E. Baker, and Ryan S. King
#Landscape Ecology https://doi.org/10.1007/s10980-023-01706-x

#Don Weller, wellerd@si.edu, Smithsonian Environmental Research Center, Edgewater, MD  21037
#last tested with Windows 10 RStudio 2023.06.0+421 "Mountain Hydrangea"
#and R version 4.3.0 (2023-04-21 ucrt) "Already Tomorrow"
#June 2023


#The code proceeds in 6 steps

#1  Read a .csv file that contains info on the models to be fit (identifiers, name, weighting equations, weighting parameters)
#   into a data frame called models  

#2  create a function called fitOne that uses non-linear optimization to fit a single model using multiple optimization algorithms,
#   then apply the function to the models data frame.  The fitted models are saved in a list called modlist

#3  create a function called getBest that does several things for each model: identify the  optimal fit among algorithms for each model:  calculate
#   model fit statistics, save the fitted parameters, calculate their standard errors, and calculated model-predicted values and residuals.
#   Then apply the function to the list modlist.
#   results are saved in a list of lists (called bestModsList), with one 6-element list
#   for each model.  The 6 elements are: the original model specification, the fit statistics, the fitted parameters, the coefficients
#   with their standard errors, predictions, and residuals).

#4  convert bestModsList into a "results" data frame in which each row gives the fitted results for one model (predictions and residuals omitted)

#5  create a function called buildObsRes that operates on bestModlist to save for each observation of each model information on
#   source proportion, weighted proportion, PCB concentration, predicted PCB value, residual value, source strength, uptake,
#   relative uptake, the ratio of weigthed to unweighted source land proportion, and the difference weighted source proportion - unweighted.
#   Then apply the function to bestModList to produce the data frame obsRes.  Add to that data frame a "pseudoobservation" that is the aggregate
#   of the the real observations.

#6  create a function called buildObsBand to apply to bestModList.  It creates a data frame with a row for each model, observation, and distance
#   band.  That df contains data for each band on actual source area, actual total area, the numerator weighting, the denominator weighting,
#   weighed source area, weighted total area,  and the sums across bands of source area, total area, weighted source area, and weighted total
#   area for the observation.  It also provides variables for realized weighting, differences and ratios of comparing weighted delivered materials
#   to unweighted, and areas, proportions, and delivered materials accumulated with distance.  The function is applied to bestModList
#   to produce the data frame obsBand.


library(plyr)
library(optimx)

#get functions from other R files
source("DistWeightFunctions.R")
source("AuxillaryFunctions.R")

#read in the file with the list of models to fit for the model comparison
#the list also includes weighting functions, the parameters of those functions,
#and starting values for the parameters
models0 <- read.csv("modlist.csv",stringsAsFactors=F)
#following line is needed because if no rows have text for denWstart, all rows will be NA; this changes NA to text
models0$denWstart <- ifelse(is.na(models0$denWstart),"",models0$denWstart)
#prepend model number to model name, helps order graphs later
models0$numname <- paste0(formatC(models0$modnum,width=2,format="d",flag="0"), models0$name)

##uncomment these two lines of code to delete the 2 systems with highest pcbs
###pcb <- subset(pcb,subset=(id!=4 & id!=32))
###ad4 <- subset(ad4,subset=(id!=4 & id!=32))
###

models <- models0 #all models
#models <- models0[1:4,]  #select subset for debugging


#function to use optimx to fit a model in a one row data frame 
#returns results in a list of three elements: the calling row of the models data frame and 2 optimx data frames
#the first one has the fits without limits, the second has the fits with the specified lower limits
fitOne <- function(modrow) {
  #construct string of all starting parameters
  partext<-modrow$linParStart
  if (modrow$numWstart!="") partext<-paste0(partext, ",", modrow$numWstart )
  if (modrow$denWstart!="") partext<-paste0(partext, ",", modrow$denWstart )
  #execute a command to define the starting parameters as a named vector
  modpar <- eval ( parse( text=paste0( "c(", partext,")" ) ) )
  # fit the model without limits
  print(paste("Model", modrow$numname, "no limits"))
  m1NL <- optimx(par=modpar,fn=mySSR, lower=NULL, hessian=T, control=list(all.methods=T,save.failures=T),
             predfun=myPred, numWeqn=modrow$numWexp, denWeqn=modrow$denWexp,
             observed=pcb$pcb, id=ad4$id, dist=ad4$d, narea=ad4$cc, darea=ad4$tc, allFlag=F )
  #execute a command that uses the text from models to set lower limits
  lowlim <- eval( parse(text=modrow$parLowLim ) )
  #fit the model with limits
  print(paste("with limits"))
  m1L <- optimx(par=modpar,fn=mySSR, lower=lowlim, hessian=T, control=list(all.methods=T,save.failures=T),
            predfun=myPred, numWeqn=modrow$numWexp, denWeqn=modrow$denWexp,
            observed=pcb$pcb, id=ad4$id, dist=ad4$d, narea=ad4$cc, darea=ad4$tc, allFlag=F )
  return( list(modrow=modrow,m1NL=m1NL,m1L=m1L) )
}
#end of function fitOne
#
#use alply function to apply the above function to each row of the models data frame
modlist <- alply(models,1,fitOne)
momodWarnings <- warnings()
modlistSave <- modlist  #keep a copy


#function to pick the best optimization of each model and build an output list of info on 
#that  optimization and its predictions
#function operates on one list element of the hierarchical list modlist
#an returns list with modrow (df), bestpar (named vect), bestopt (df), predicted (vec), and residuals
getBest <- function(listelement) {
  #recover the model information and optimizations
  modrow <- listelement$modrow;   npar <- modrow$npar  #number of parameters
  m1NL  <- listelement$m1NL
  m1L  <- listelement$m1L
  #add limits indicator and the optimization method from the row.names
  #also add a flag indicating whether or not the method provided a Hessian matrix
  #also whether or not a Hessian matrix was computed
  #the Hessian matrix is called nhatend and is hidden in the data frame attributes 
  m1NL <- within( m1NL, {limits <- F; method <- row.names(m1NL); hasHess <- !is.na(attr(m1NL,"details")[,"nhatend"])} )
  m1L  <- within( m1L,  {limits <- T; method <- row.names(m1L);  hasHess <- !is.na(attr(m1L,"details")[,"nhatend"])}  )
  #combine with and without limits solutions into one data frame #and add variable for limits or not
  mm1 <- rbind( m1NL, m1L  )
  
  #now work to pick out the model with the minimum SSR
  #first, keep only solutions that converged
  mm1a <- subset(mm1, subset=(convcode==0))
  #second, get rid of solutions that gave negative values for any parameter other than b0
  #do this by finding the minimum value of params 2:npar (omitting param 1, b0)
  if (npar==2) minpar<-min(mm1a[,2])  else minpar<-apply(mm1a[,2:npar],1,min)  
  mm1b <- mm1a[minpar>=0,]   #keep rows where parameters 2:npar are not negative
  ##Third, get rid of Rcgmin solutions; Rcgmin can quit with the message " No acceptable point -- exit loop"
  ##and then return the unadjusted initial parameters with the wrong SSR
  mm1c <- subset(mm1b,subset=(method!="Rcgmin"))
  #fourth, only keep the results for the method with minimum SSR
  minSSR <- min(mm1c$value)
  mm2 <- mm1c[mm1c$value<=1.00001*minSSR,]  #include those = min within numerical precision
  #Fifth, keep solutions that provide a Hessian matrix (if there are any)
  mm2a <- subset(mm2,subset=hasHess==T)
  #if none provide a hessian matrix, keep the ones that didn't
  if (nrow(mm2a)==0) { mm2a<-mm2; print(paste("No Hessian ", modrow$numname, modrow$parSet, mm2a$method)) }
  mm3 <- mm2[1,]   #if there are more than one solutions with min SSR, arbitrarily keep the first one
  
  #now that best solution is selected, add the R2 and AICc;  in AICc calculation, add 1 to npar for sigma**2
  mm3 <- within(mm3, { R2<-myR2SSR(pcb$pcb,value); AICc<-myAICcSSR(value,length(pcb$pcb),npar+1) } )
  
  #construct parameter vector
  pardf <- mm3[,1:npar]   #df with just the best parameters
  #convert from data frame to named list
  bestpar <- as.vector(t(pardf)); names(bestpar)<-names(pardf)   #parameters as named vector
  
  #now work on the SEs for the parameters
  #get the Hessian matrix for selected best model, it's in the attributes of the m1L or m1NL data frame
  #build a command to get the correct Hessian from the attribute of m1L of m1NL
  Hcmd<-paste0("attr(", ifelse(mm3$limits,"m1L","m1NL"), ",\"details\")[,\"nhatend\"]$'",mm3$method,"'" )
  H<-eval(parse(text=Hcmd))          #execute the command to get the Hessian
  #call the function that creates a coefficient matrix from parameters, Hessian, SSR, and nobs
  cm <- coefFromHess(bestpar,mm3$value,H,length(pcb$pcb))
  #add coefficients and statistics to m3 data frame
  #flatten coefficient matrix by columns then rows;  drop col 1--parameters are already in mm3
  cf <- as.vector(t(cm[,2:4])) 
  names(cf) <- paste0(rep(row.names(cm),each=3), c("SE","T","P"))  #assign names
  mm3 <- cbind(mm3,t(cf))
  
  #calculate a prediction for all subestuaries lumped together, add it to mm3
  mm3$predAll <- myPred(bestpar, numWeqn=modrow$numWexp, denWeqn=modrow$denWexp,
       id=ad4$id, d=ad4$d, narea=ad4$cc, darea=ad4$tc, allFlag=T )
  
  #finally, get predictions for each observation
  predictions <-myPred(bestpar, numWeqn=modrow$numWexp, denWeqn=modrow$denWexp,
       id=ad4$id, d=ad4$d, narea=ad4$cc, darea=ad4$tc, allFlag=F )
  residuals <- pcb$pcb-predictions
  
  #return a 6 element list describing the best model
  mm3$method <- as.character(mm3$method)
  mlist <- list(modrow=modrow,modstats=mm3,modpar=bestpar,modcoef=cm,predictions=predictions,residuals=residuals)
  return(mlist)
}
#end of function getBest
#apply function getBest to list of models to select the best optimization for each model
bestModsList <- llply(modlist,getBest)




#build data frame with performance measures as well as standard errors, T statistics, and P values (signinficance levels) of parameters for each model
bestdf <- function(le) { cbind(le$modrow,le$modstats) }
bestMods <- ldply(bestModsList,bestdf,.id=NULL)
#add deltaAICc and model weights
bestMods <- within(bestMods, { deltaAICc <- AICc-min(AICc); wi <- exp(-deltaAICc)/sum(exp(-deltaAICc))  })
#reorder the variables and drop a few
results <- subset(bestMods,select=c(modnum,name,numname,parSet,method,limits,hasHess,fevals,niter,
      value,R2,AICc,deltaAICc,wi,predAll,npar,parnames,
      b0,b0SE,b0T,b0P,b1,b1SE,b1T,b1P,t,tSE,tT,tP,td,tdSE,tdT,tdP,p,pSE,pT,pP,pd,pdSE,pdT,pdP,
      k,kSE,kT,kP,kd,kdSE,kdT,kdP,z1,z1SE,z1T,z1P,z2,z2SE,z2T,z2P))    


#now assemble obs-by-obs results (predictions, residuals, effective proportions, sources, uptakes) into data frame called obsRes
#function to do this for one model
buildObsRes <- function(le) {
  modrow <- le$modrow
  modpar <- le$modpar
  residuals  <- data.frame(residual=le$residuals, id=names(le$residuals) )
  predictions <- data.frame(predicted=le$predictions, id=names(le$predictions) )
  #extract identifying info on this model
  modinf <- subset(modrow,select=c(modnum,name,numname,parSet))
  #start with areas summarized by subestuary & calculate source proportion
  rr1 <- adsum[c("id","cc","tc")]
  rr1$prop <- with(rr1,cc/tc)      #calculate unweighted proportion of source land
  #merge in the observed data
  rr2 <- merge(rr1,pcb[c("id","pcb")],by="id")
  #add the vectors of model predictions and residuals
  rr3 <- merge(rr2,predictions,by="id")
  rr4 <- merge(rr3,residuals,by="id")
  #now calculate effective proportion, source strength, uptake, and relative uptake
  rr5 <- within(rr4, {
    #use the fitted values for b0 and b1 to recover the effective source land proportion from the predictions
    effprop <- (predicted-modpar["b0"])/modpar["b1"]
    #calculate the source strength of pcb from the unweighted source land proportion
    source <- modpar["b0"] + modpar["b1"]*prop
    #and then the amount of uptake
    uptake <- source-predicted
    #and uptake as a proportion of source
    reluptake <- uptake/source
    propRatio <- effprop / prop
    propDiff <- effprop - prop
    relPropDiff <- 100*propDiff/prop
  })
  #select vars and reorder
##  rr6 <- rr5[c("id","prop","effprop","pcb","predicted","residual","source","uptake","reluptake")]
  rr6 <- subset(rr5,select=c(id,prop,effprop,pcb,predicted,residual,source,uptake,reluptake,propRatio,propDiff))
  rr7 <- cbind(modinf,rr6,row.names=NULL ) 
  return(rr7)
}
#end function buildObRes
#
#now apply the function to get data frame of ObsRes for all models
obsRes <- ldply(bestModsList,buildObsRes,.id=NULL)

#create a pseudo estuary (id=0) that aggregates all the sampled estuaries by distance bands
ad4agg <- ddply(ad4,.(d),colwise(sum,c("cc","hc","lc","tc")))
ad4agg$id <- 0   #assign id of 0
#combine the pseudo subestuary with the sampled subestuaries
ad5 <- rbind(ad4,ad4agg)
ad5 <- ad5[order(ad5$id,ad5$d),]; row.names(ad5) <- NULL  #make sure in proper order

####le <- bestModsList[[1]]; allFlag=F
#now create a data frame called obsBand with information on each distance band
#also save band info on areas, weights, and weighted areas
buildObsBand <- function(le,allFlag) {
  modrow <- le$modrow
  modpar <- le$modpar
  #select model info
  modinf <- subset(modrow,select=c(modnum,name,numname,parSet))
  #get the distance and weighting information for each distance band for this model
  r2 <- bandWeights( par=modpar, numWeqn=modrow$numWexp, denWeqn=modrow$denWexp,
        id=ad5$id, d=ad5$d, narea=ad5$cc, darea=ad5$tc )
  #sum unweighted and weighted areas across bands within each subestuary
  sumsr2 <- ddply(r2, .(id), colwise(sum,c("narea","darea","nwtd","dwtd")))
  names(sumsr2) <- c("id","nareaSum","dareaSum","nwtdSum","dwtdSum")
  r3 <- merge(r2,sumsr2,by="id",all=T)
  #calculate some new quantities
  r4 <- within(r3, {
    logd <- log10(d)                 #for plotting later
    dkm <- d/1000
    prop    <- narea/dareaSum        #actual land use proportion
    propWtd <- nwtd/dwtdSum          #weighted (effective) land use proportion
    source  <- modpar["b1"]*prop     #unweighted material
    delivrd <- modpar["b1"]*propWtd  #weighted material
    propDiff <- propWtd - prop
    propRatio <- propWtd/prop        #NA when prop=0
    b1Unwtd <- results[results$name=="Unwtd","b1"] #slope b1 for unweighted model
    delivrdUnwtd <- prop*b1Unwtd     #delivered from this band for unweighted model
    delivrdDiff <- delivrd - delivrdUnwtd
    delivrdRatio <- delivrd/delivrdUnwtd
    relDelivrdDiff <- 100*delivrdDiff/delivrdUnwtd
    #realized weighting r=delivered/delivered in unweighted model (for each band)
    #the realized weighting accounts for actual distribution, denominator weighting, and b1 from model
    realzdWgt <- delivrd / delivrdUnwtd   #note:  NA when delivrdUnwtd=0
   })
  r5 <- ddply(r4, .(id),  colwise(cumsum,c("narea","darea","nwtd","dwtd","prop","propWtd","source","delivrd")) )
  names(r5) <- c("id","nareaCum","dareaCum","nwtdCum","dwtdCum","propCum", "propWtdCum", "sourceCum", "delivrdCum")  
  r6 <- cbind(r4,subset(r5,select=-id))  #use cbind because merge fails to allocate memory 
  #note: this excludes the b0 part, but that won't change the rankings  
  r7 <- ddply(r6, .(id),  colwise(function(x) {x/max(x)},"delivrdCum") )
  names(r7) <- c("id","delivrdCumFrac")
  r8 <- ddply(r6, .(id),  colwise(function(x) {x/sum(x)},"delivrd") )
  names(r8) <- c("id","delivrdFrac")
  r9 <- cbind(r6,subset(r7,select=-id),subset(r8,select=-id))
  r10 <- cbind(modinf,r9,row.names=NULL)
  r11 <- r10[order(r10$modnum,r10$id,r10$d),]; row.names(r11) <- NULL
  return(r10)
}
#end of function buildObsBand
#now apply the function to all models to build obsBand
obsBand <- ldply(bestModsList,buildObsBand,.id=NULL,.progress="text",allFlag=F)


