#3Tables.R

#Create the tables for this published paper:
#New methods for quantifying the effects of catchment spatial patterns on aquatic responses
#by Donald E. Weller, Matthew E. Baker, and Ryan S. King
#Landscape Ecology https://doi.org/10.1007/s10980-023-01706-x

#Don Weller, wellerd@si.edu, Smithsonian Environmental Research Center, Edgewater, MD  21037
#last tested with Windows 10 RStudio 2023.06.0+421 "Mountain Hydrangea"
#and R version 4.3.0 (2023-04-21 ucrt) "Already Tomorrow"
#June 2023

library(plyr)
library(TAM)

#tables will be saved as .csv files to the subdirectory called FiguresTables
#if the directory does not exist yet, create it
if (!dir.exists("FiguresTables"))  dir.create("FiguresTables")

#function compTable
#used to make one table for a specific comparison and write it to a .csv file
compTable <- function(filename,modsel,parsel,outvar,results,writeFlag) {
  tab <- subset( results,subset=( (name %in% modsel) & (parSet %in% parsel ) ) )
  #recalculate deltaAICc and wi just for this subset
  tab <- within(tab, { deltaAICc <- AICc-min(AICc); wi <- exp(-deltaAICc/2)/sum(exp(-deltaAICc/2))  })
  tab2 <- tab[,outvar]
  if (writeFlag) write.csv(tab2,paste0("./FiguresTables/",filename,".csv"),row.names=F)
  return(tab2)
}

#Table1
#Final set of double weighted models performance and parameters (include SE, T, and P)
modsel <-c("Unwtd","StepDbl","PieceDbl","InvFitDbl(d+1)","ExpFitDbl")
parsel = c(1,2)
nn1 <- data.frame( name=c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)"),
                   weightingFunction=c("Unweighted","Step","Linear","Exponential","Power"),stringsAsFactors=F )
results2 <- merge(results,nn1,by="name")
results2 <- results2[order(results2$numname),]
outvar <- c("numname","weightingFunction","R2","AICc","deltaAICc","wi","b0","b0SE","b1","b1SE","t","tSE","k","kSE","p","pSE")
finSet <- compTable("Table1",modsel,parsel,outvar,results2,T)
rm(modsel,parsel,nn1,results2,outvar,finSet)

#Table2
#Comparesingle vs. Double weighting
modsel <-c("Unwtd","StepSgl","PieceSgl","ExpFitSgl","InvFitSgl(d+1)","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
parsel <-c(1,2)
nn1 <- data.frame( name=c("Unwtd","StepSgl","PieceSgl","ExpFitSgl","InvFitSgl(d+1)","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)"),
                   weightingFunction=c("Unweighted","Step","Linear","Exponential","Power","Step","Linear","Exponential","Power"),
                   weightType=c("Unweighted",rep("Single",4),rep("Double",4) )
                   ,stringsAsFactors=F )
results2 <- merge(results,nn1,by="name")
results2 <- results2[order(results2$numname),]
#keep only the second parameter set when two were attempted with single weighting
results2 <- subset( results2, subset=!(parSet==1 & name %in% c("StepSgl","PieceSgl","GenThrshFitSgl","GenThrshFitSep") ) )
outvar <- c("numname","weightType","weightingFunction","R2","AICc","deltaAICc","b0","b1","t","k","p")
sglVSdbl <- compTable("Table2",modsel,parsel,outvar,results2,F)
#recalculate deltaAICc relative to unweighted model rather than minimum AICc model
sglVSdbl <- within( sglVSdbl, { deltaAICc<-AICc-sglVSdbl[weightType=="Unweighted","AICc"] } )
write.csv(sglVSdbl,"./FiguresTables/Table2.csv",row.names=F)
rm(modsel,parsel,nn1,results2,outvar,sglVSdbl)


#Table3
#Compare double vs. separate weighting
modsel <-c("StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)","StepSep","PieceSep","ExpFitSep","InvFitSep(d+1)")
parsel <-c(1,2)
outvar <- c("numname","weightingFunction","weightType","R2","AICc","deltaAICc","wi","b0","b1","t","td","k","kd","p","pd")
nn1 <- data.frame( name=c("StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)","StepSep","PieceSep","ExpFitSep","InvFitSep(d+1)"),
                   weightingFunction=c("Step","Linear","Exponential","Power","Step","Linear","Exponential","Power"),
                   weightType=c(rep("Double",4),rep("Separate",4) )
                   ,stringsAsFactors=F )
results2 <- merge(results,nn1,by="name")
results2 <- results2[order(results2$numname),]
qq1 <- compTable("Table3",modsel,parsel,outvar,results2,F)
#change AICc and wi in above to pairwise
deltaWiGroup <- function(dataframe) {
  df <- dataframe
  df <- within( df, {deltaR2<-R2-min(R2); deltaAICc<-AICc-min(AICc); wi <-exp(-deltaAICc/2)/sum(exp(-deltaAICc/2))} )
  return(df)
}
qq2 <- ddply(qq1, .(weightingFunction), deltaWiGroup )
outvar2 <- c("numname","weightType","R2","deltaR2","AICc","deltaAICc","wi","b0","b1","t","td","k","kd","p","pd")
dblVSsep <- qq2[order(qq2$numname),outvar2]
write.csv(dblVSsep,"./FiguresTables/Table3.csv",row.names=F)
rm(modsel,parsel,outvar,nn1,results2,qq1,deltaWiGroup,qq2,outvar2,dblVSsep)
###end Table2

#TableS2
#Final set of double weighted models performance and parameters (include SE, T, and P) with general threshold
modsel <-c("Unwtd","StepDbl","PieceDbl","GenThrshFitDbl","InvFitDbl(d+1)","ExpFitDbl")
parsel = c(1,2)
nn1 <- data.frame( name=c("Unwtd","StepDbl","PieceDbl","GenThrshFitDbl","ExpFitDbl","InvFitDbl(d+1)"),
                   weightingFunction=c("Unweighted","Step","Linear","General threshold","Exponential","Power"),
                   listNum=c(1,2,3,4,5,6), stringsAsFactors=F )
results2 <- merge(results,nn1,by="name")
results2 <- results2[order(results2$listNum),]
outvar <- c("numname","weightingFunction","R2","AICc","deltaAICc","wi","b0","b0SE","b1","b1SE","t","tSE","k","kSE","p","pSE")
finSet <- compTable("TableS2",modsel,parsel,outvar,results2,T)
rm(modsel,parsel,nn1,results2,outvar,finSet)

#TableS3
#Compare fitted inverse power weighting vs. a priori assumed powers
modsel <-c("InvFitDbl(d+1)","InvSqrtDbl(d+1)","Inv1Dbl(d+1)","Inv2Dbl(d+1)")
parsel = 1
nn1 <- data.frame( name=c("InvFitDbl(d+1)","InvSqrtDbl(d+1)","Inv1Dbl(d+1)","Inv2Dbl(d+1)"),
                   weightingFunction=c("Fitted inverse","Inverse square root","Simple inverse","Inverse squared"),p2=c(NA,0.5,1,2),stringsAsFactors=F )
results2 <- merge(results,nn1,by="name")
#there are no fitted results for three models, so replace NA for p with the a prior values
results2 <- results2[order(results2$numname),]
results2[is.na(results2$p),"p"]<-results2[is.na(results2$p),"p2"]
outvar <- c("numname","weightingFunction","R2","AICc","deltaAICc","wi","b0","b1","p")
fitVSnot <- compTable("TableS3",modsel,parsel,outvar,results2,T)
rm(modsel,parsel,nn1,results2,outvar,fitVSnot)

#TableS5
#Compare different adjustments for inverse power functions
#Double vs. step
modsel <-c("InvFitDbl","InvFitDbl(d+1)","InvFitDbl(z1*d+1)","InvFitDbl(d+z2)")
parsel = 1
nn1 <- data.frame( name=c("InvFitDbl","InvFitDbl(d+1)","InvFitDbl(z1*d+1)","InvFitDbl(d+z2)"),
                   distVar=c("d","d + 1","z1 d + 1","d + z2"),stringsAsFactors=F )
results2 <- merge(results,nn1,by="name")
#there are no fitted results for three models, so replace NA for p with the a prior values
results2 <- results2[order(results2$numname),]
outvar <- c("numname","distVar","R2","AICc","deltaAICc","wi","b0","b1","p","z1","z1SE","z2","z2SE")
invAdj <- compTable("TableS5",modsel,parsel,outvar,results2,T)
rm(modsel,parsel,nn1,results2,outvar,invAdj)

#TableS8andS9
#how weighting changes source and total land sums, overall land proportions, and the factor J (see equation 19 in paper)
#also, distances needed to supply set percentages of total material delivery by subestuary and model
#
#sums across all distances for each model by subestuary
obsBandSums <-  ddply(obsBand, .(modnum,name,numname,parSet,id), colwise(sum,c("narea","darea","nwtd","dwtd","prop","propWtd","delivrd","source")) )
#add in b0 and b1 from the fitted models (in results dataframe)
obsBandSums <- merge(obsBandSums,results[,c("modnum","b0","b1")],by="modnum")
#calculate source and delivered INCLUDING the intercept b0
obsBandSums <- within(obsBandSums, {  delivrdPlusb0<-delivrd+b0; sourcePlusb0<-source+b0 })
#funtion myStats
#summarize descriptive statistics on distance distribution of delivrd by model for each subestuary
#use weighted functions from package TAM
myStats <- function(dframe) {
  d<-dframe$d; val<-dframe$delivrd
  mean<-weighted_mean(d, val )
  var<-weighted_var(d, val, method="unbiased" )
  sd<-weighted_sd(d, val, method="unbiased" )
  skew<-weighted_skewness(d, val )
  kurt<-weighted_kurtosis(d, val )
  median <-weighted_quantile(d, val, probs=0.5 )
  quantiles <-data.frame( t(  weighted_quantile(d, val, probs=c(.05,.1,.25,0.5,.75,.90,.95) )  ) )
  names(quantiles) <- c("d05","d10","d25","d50","d75","d90","d95")
  return(data.frame(median,mean,var,sd,skew,kurt,quantiles))
}
#end of function myStats
#apply above function by model and id
#using subsets of models and vars reporte in the manuscript
#use the aggregate and the 4 selected subestuaries 3, 4, 29,and 32
modsel <- c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
subestsel <- c(0,3,4,29,32)
distStats <- ddply(subset(obsBand,subset=(name %in% modsel & id %in% subestsel)),.(modnum,id), myStats, .progress="text" )
obsBandSums <- merge(obsBandSums,distStats,by=c("id","modnum"))
obsBandSums <- obsBandSums[order(obsBandSums$modnum,obsBandSums$id),]; row.names(obsBandSums) <- NULL  #make sure in proper order

oB2 <- subset(obsBandSums,select=c(numname,name,id,nwtd,dwtd,propWtd,sourcePlusb0,delivrdPlusb0,d05,d10,d25,d50,d75,d90,d95))
nn1 <- data.frame( name=c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)"),
                   weightingFunction=c("Unweighted","Step","Linear","Exponential","Power"),
                   stringsAsFactors=F )
oB2 <- merge(oB2,nn1,by="name")
#convert areas from ha to km2 and calculate factor J (see equation 19 of paper)
oB2uw <- subset(oB2,subset=(name=="Unwtd"),select=c(id,nwtd,dwtd))  #extract the sums for the unweighted model
names(oB2uw) <- c("id","nu","du")
oB3 <- merge(oB2,oB2uw,by=c("id"))
oB3 <- within( oB3, { nwtd <- nwtd / 100; dwtd <- dwtd / 100; nu <- nu/100; du <- du/100; J <- du/dwtd } )
oB3 <- oB3[order(oB3$id,oB3$numname),]   #put in order for table (by subestuary, then model)
#make better watershed names for table
oB3$id <- as.character(oB3$id)
oB3[oB3$id!="0","id"] <- paste("Watershed", oB3[oB3$id!="0","id"] )
oB3[oB3$id=="0","id"] <- "Aggregate"
oB4 <- subset(oB3,select=c(id,numname,weightingFunction,nwtd,dwtd,propWtd,J))
oB5 <- subset(oB3,select=c(id,numname,weightingFunction,d05,d10,d25,d50,d75,d90,d95))
#write out table
write.csv(oB4,"./FiguresTables/TableS8.csv",row.names=F)  #Table S8
write.csv(oB5,"./FiguresTables/TableS9.csv",row.names=F)  #Table S9
rm(obsBandSums,myStats,modsel,subestsel,oB2,nn1,oB2uw,oB3,oB4,oB5)
###end of TablesS8andS9

