#4Figures.R

#Create the figures for this published paper:
#New methods for quantifying the effects of catchment spatial patterns on aquatic responses
#by Donald E. Weller, Matthew E. Baker, and Ryan S. King
#Landscape Ecology https://doi.org/10.1007/s10980-023-01706-x

#Don Weller, wellerd@si.edu, Smithsonian Environmental Research Center, Edgewater, MD  21037
#last tested with Windows 10 RStudio 2023.06.0+421 "Mountain Hydrangea"
#and R version 4.3.0 (2023-04-21 ucrt) "Already Tomorrow"
#June 2023

library(plyr)
library(ggplot2)
library(grid)
my_theme <- theme_bw(base_size=9) + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
  theme(axis.text= element_text(color="black")) +
  theme(axis.title.x = element_text(size = 9), axis.text.x = element_text(size = 8),
        axis.title.y = element_text(size = 9), axis.text.y = element_text(size = 8)  ) +
  theme(strip.background=element_rect(fill=NA))
my_theme <- my_theme + theme(panel.spacing = unit(0.8, "lines"))
theme_set(my_theme)

#figures will be saved as .tif files to the subdirectory called FiguresTables
#if the directory does not exist yet, create it
if (!dir.exists("FiguresTables"))  dir.create("FiguresTables")

#function to save a plot file in lzw-compressed TIFF format using specified file name as well was width and height in inches
saveIt <- function(name,pt,wid,hgt) {
  tiff( paste0( ".\\FiguresTables\\", name, ".tif" ), width=wid,height=hgt,units="in",res=600,compression="lzw",restoreConsole=T);
  print(pt); dev.off()
  }

# Figure1
#distance distributions of all land and commercial land for aggregate and 4 selected subestuaries
#kludge lines at y=0 extend beyond domain of data, fix by putting gray line from upper
#also use for adding subestuary name
subestVec <- c(0,29,32)  #aggregate + 4 individual watersheds
kk <- data.frame(id=subestVec, ndmax=c(45.85,8.96,29.31), ddmax=c(47.18,9.12,29.47), maxmax=rep(47.18,3),
                 xv=rep(45,3), yv=c(140,5.9,7.5),  idtxt=c("Aggregate","29","32"), stringsAsFactors=F )
obsBandSel<-subset(obsBand,subset=(modnum==1 & id %in% subestVec))
obsBandSel <- merge(obsBandSel,kk,by="id")
pt <- ggplot(data=subset(obsBandSel[obsBandSel$modnum==1,],subset=(id %in% c(0,29,32))), aes(x=dkm,weight=darea/100)) + 
  geom_histogram(binwidth=.2,color="gray80",fill="gray80") +
  geom_histogram(aes(x=dkm,weight=narea/100),color="black",fill="black",binwidth=.2) +
  geom_segment(aes(x=ndmax+.1,y=0,xend=maxmax,yend=0),color="gray80") +
  geom_segment(aes(x=ddmax+.1,y=0,xend=maxmax+.1,yend=0),color="white") +
  facet_grid(rows=vars(id),scale="free") +
  geom_text(aes(x=xv,y=yv,label=idtxt),size=2.5, hjust=1, fontface="plain") +
  theme(strip.background = element_blank(), strip.text.x = element_blank(), strip.text.y = element_blank()) +
  ylab(expression("Area km"^2)) + xlab("Distance km"); pt
saveIt("Figure1",pt,3,3.6)
rm(subestVec,obsBandSel,kk,pt)  #cleanup
#bin width of .2 (200 meters) with 20 km max gives 100 bins
#used the following to figure out where to draw white lines in above kludge
#qq <- obsBandSel[obsBandSel$modnum==1 & obsBandSel$id==0,]; max(qq[qq$narea>0,"d"]); max(qq$d)
###end of Figure1

#Figure2
#predicted pcb vs. weighted proportion for selected models
#shows how weighting shifts proportion to effective proportion
#black points are data pcb vs effective proportion (from weighting)
#two subestuaries with pcb>300 are highlighted in red
#gray points are pcb vs actual proportion
#gray segments connect actual to effective proportion, showing shifts by subestuary [maybe add arrows?]
#blue line is regression of pcb vs effective proportion
modVec <- c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
#set up a vectors for translating legend labels
numname.labs <- c("Unweighted","Step","Linear","Exponential","Power")
names(numname.labs) <- c("01Unwtd","13StepDbl","15PieceDbl","19ExpFitDbl","22InvFitDbl(d+1)")
subestVec <- c(3,4,29,32)
obsResSel <- subset(obsRes,subset=(name %in% modVec) )
obsResLab <- subset(obsResSel,subset=(id %in% subestVec))
#plots of model predictions, residuals for subestuaries
pt <- ggplot(obsResSel,aes(x=effprop,y=pcb)) + facet_wrap(vars(numname),ncol=1,labeller=labeller(numname=numname.labs)) +
  geom_smooth(method=lm) + 
  geom_segment(aes(x=prop,y=pcb,xend=effprop,yend=pcb),color="gray80") +
  geom_point(aes(x=prop,y=pcb),color="gray80") +  geom_point() +
  geom_point(data=obsResLab,color="red") + 
  geom_text(data=obsResLab,aes(x=effprop,y=pcb,label=id), size=2.5,  nudge_x=c(0,.015,.005,.02), nudge_y=c(180,70,-80,0)  ) +
  xlab("Weighted commercial land proportion") +
  ylab("Observed PCB ng/g"); pt
saveIt("Figure2",pt,3,6)
rm(modVec,subestVec,obsResSel,obsResLab,numname.labs)
##characteristics of selected subestuaries
##0   pseudo estuary aggregate of all observed subestuaries
##29  largest drop from prop to eff prop, ~ 100 ng/g PCB
##32  largest increase from prop to eff prop, also hightest [pcb]=623
##3   no visible change in proportion (see Fig. 1), 3rd highest conc pcb=101
##4   increase in prop to eff prop for 3 mods, decrease for ExpFitDbl, no change for InvFitDbl(d+1); 2nd highest pcp=310
###end of Figure2

#Figure3
#show four measures of material source strength vs distance for the aggregate of all study watersheds
#measures are stacked vertically, 5 models shown across horizontally
modVec <- c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
#set up a vectors for translating legend labels
numname.labs <- c("Unweighted","Step","Linear","Exponential","Power")
names(numname.labs) <- c("01Unwtd","13StepDbl","15PieceDbl","19ExpFitDbl","22InvFitDbl(d+1)")
#use models in modVec and only the Aggregate subestuary (id==0)
obsBandSel<-subset(obsBand,subset=(name %in% modVec & id==0 ))
binw <- 0.09        #width of histogram bins is 90 meters
dmax <- 3           #x axis max, only show up to 3 km to focus on shorter distances
#weighted commercial land proportion gb vs distance
pt1 <- ggplot(obsBandSel,aes(x=dkm,weight=propWtd)) + geom_histogram(binwidth=binw,boundary=binw) + coord_cartesian(xlim=c(0,dmax)) +
  facet_grid(cols=vars(numname),scale="free",labeller=labeller(numname=numname.labs)) + 
  ylab("gb") + xlab("Distance (km)") +
  theme(axis.title.x=element_blank(), axis.text.x=element_blank() ); pt1
#following just verifies that sums of the histogram y data is the overall weighted proportion for the entire study area
qq <- ggplot_build(pt1)
qq1 <- qq[1]
qq2 <- as.data.frame(qq1[1])
qq3 <- ddply(qq2, .(data.PANEL),colwise(sum,"data.y") )
qq3$data.y
###weighted source delivery hb against distance
pt2 <- ggplot(obsBandSel,aes(x=dkm,weight=delivrd)) + geom_histogram(binwidth=binw,boundary=binw) + coord_cartesian(xlim=c(0,dmax)) +
  facet_grid(cols=vars(numname),scale="free",labeller=labeller(numname=numname.labs)) + 
  ylab("hb") + xlab("Distance (km)") +
  theme(axis.title.x=element_blank(), axis.text.x=element_blank(), strip.text=element_blank() ); pt2
#delivrd ratio hb/cb  gb/fb (realized weight)
pt3 <- ggplot(obsBandSel,aes(x=dkm,y=delivrdRatio)) +
  geom_hline(yintercept=1,color="gray60",linetype="dashed") +  geom_hline(yintercept=0,color="gray80",linewidth=0.3) +
  geom_line( data=subset(obsBandSel,subset=(delivrdRatio>=1)),aes(x=dkm,y=delivrdRatio),color="black" ) +
  geom_line( data=subset(obsBandSel,subset=(delivrdRatio<1)),aes(x=dkm,y=delivrdRatio),color="red" ) +
  coord_cartesian(xlim=c(0,dmax)) +
  facet_grid(cols=vars(numname),scale="free",labeller=labeller(numname=numname.labs)) + 
  ylab("hb / cb") + xlab("Distance (km)") +
  theme(axis.title.x=element_blank(), axis.text.x=element_blank(), strip.text=element_blank()  ); pt3
#hb-cb, weighted delivery - unweighted
#create information to color bars red and black
deliveryobsBandSel <- within( obsBandSel, {
  nonNeg <- ifelse(delivrdDiff<0,0,delivrdDiff)
  negs0 <-  ifelse(delivrdDiff<0,delivrdDiff,0)
  negs0 <-  ifelse(delivrdDiff<0,delivrdDiff,0)
  negsNA <- ifelse(delivrdDiff<0,delivrdDiff,NA)
})
pt4 <- ggplot(deliveryobsBandSel,aes(x=dkm,weight=nonNeg)) + coord_cartesian(xlim=c(0,dmax)) + 
  geom_histogram(data=deliveryobsBandSel,aes(x=dkm,weight=negsNA),binwidth=binw,boundary=binw,color="red",fill="red") + 
  geom_histogram(binwidth=binw,boundary=binw,color="black",fill="black") +
  geom_hline(yintercept=0,color="gray80",linewidth=0.3) +
  facet_grid(cols=vars(numname),scale="free",labeller=labeller(numname=numname.labs)) +
  xlab("Distance (km)") + ylab("hb - cb") + 
  theme(strip.text=element_blank() ); pt4
#now put the four panels into into one page using grid
#note:  width and x kludged to fix vertical misalignment of x axes
vp1 <- viewport(height=.25,width=.985,y=0,x=0.015,just=c("left","bottom"))
vp2 <- viewport(height=.25,width=.985,y=.25,x=0.015,just=c("left","bottom"))
vp3 <- viewport(height=.25,width=.985,y=.50,x=0.015,just=c("left","bottom"))
vp4 <- viewport(height=.25,width=1,y=.75,x=0,just=c("left","bottom"))
#
tiff(".\\FiguresTables\\Figure3.tif", width=6,height=6,units="in",res=600,compression='lzw',restoreConsole=T);
print(pt1,vp=vp4); print(pt2,vp=vp3); print(pt3,vp=vp2); print(pt4,vp=vp1);
dev.off()
rm(modVec,numname.labs,obsBandSel,binw,dmax,qq,qq1,qq2,qq3,pt1,pt2,pt3,pt4,vp1,vp2,vp3,vp4)
###end Figure3

#Figure4
#cumulated delivery fraction by model on one plot, aggregate watershed only
modVec <- c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
#use models in modVec and only the Aggregate subestuary (id==0)
obsBandSel<-subset(obsBand,subset=(name %in% modVec & id==0 ))
#create name2 vector to order the colors in the legend
qq1 <- data.frame( name=c("InvFitDbl(d+1)","ExpFitDbl","PieceDbl","StepDbl","Unwtd"),
                   name2=c("1Power","2Exponential","3Linear","4Step","5Unweighted") )
obsBandSel <- merge(obsBandSel,qq1,by="name")
pt <- ggplot(obsBandSel,aes(x=d,y=delivrdCumFrac,color=name2)) + coord_cartesian(xlim=c(0,3000)) + 
  geom_hline(yintercept=0,color="gray90") +
  geom_hline(yintercept=.25,color="gray90") +
  geom_hline(yintercept=.5,color="gray90") +
  geom_hline(yintercept=.75,color="gray90") +
  geom_hline(yintercept=1,color="gray90") +
  geom_line() +
  xlab("Distance (m)") + ylab("Cumulative delivery fraction") +
  theme(legend.title = element_blank()) +
  theme(legend.position = c(0.6, 0.62)) + 
  theme(legend.background = element_rect(fill="transparent")) +
  scale_color_manual(values=c("red","green","blue","magenta","black"),labels=c("Power","Exponential","Linear","Step","Unweighted"))+
  theme(legend.key.height = unit(.25, "cm")); pt
saveIt("Figure4",pt,3,3)
rm(modVec,obsBandSel,qq1,pt)
###end Figure4

#Figure5
#changes in PCB delivery with distance,  weighted models compared to unweighted
#as above, but with only 2 individual subestuaries
modVec <- c("StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")   #note unweighted model omitted
subestVec <- c(0,29,32)
#set up a vectors for translating legend labels
numname.labs <- c("Step","Linear","Exponential","Power")
id.labs <- c("Aggregate","29","32")
names(id.labs) <- c("0","29","32")
names(numname.labs) <- c("13StepDbl","15PieceDbl","19ExpFitDbl","22InvFitDbl(d+1)")
obsBandSel<-subset(obsBand,subset=(name %in% modVec & id %in% subestVec))
pt <- ggplot(subset(obsBandSel,subset=(id %in% c(0,29,32))),aes(x=dkm,weight=delivrdDiff)) + coord_cartesian(xlim=c(0,10)) + 
  geom_histogram(binwidth=.1) +
  geom_histogram(data=subset(obsBandSel[obsBandSel$delivrdDiff<0,],subset=(id %in% c(0,29,32)) ),binwidth=.1,color="red",fill="red") + 
  geom_hline(yintercept=0,color="gray") +
  facet_grid(rows=vars(id),cols=vars(numname),scale="free",labeller=labeller(id=id.labs,numname=numname.labs)) +
  xlab("Distance (km)") + ylab("Delivered PCB - delivered PCB from unweighted model"); pt
saveIt("Figure5",pt,6,3.6)
remove(modVec,subestVec,id.labs,obsBandSel,pt)
###end Figure5

#FigureS1
#plot generalized weighing function for a few p values
#generate the curves
genthrsh <- function(d,dthresh,p) {ifelse(d<dthresh,1-(d/dthresh)**p,0)}
dmax <- 1500
dthresh <- 1000
p0 <- c(.2,.5,1,2,5,100)
d0 <- seq(0,dmax,dmax/100)
p <- rep(p0,length(d0))
p <- p[order(p)]
d <- rep(d0,length(p0))
dfgt <- data.frame(p,d)
dfgt$w <- genthrsh(dfgt$d,dthresh,dfgt$p)
#make a data set for labeling the curves wit the p values
dfgtlab <- dfgt[dfgt$w<0.5,]
labmax <- ddply( dfgtlab,.(p), colwise(max,"w") )
names(labmax) <- c("p","wmax")
dfgtlab2 <- merge(dfgtlab,labmax,by="p")
dfgtlab3 <- subset(dfgtlab2, subset=(w==wmax))
dfgtlab4 <- dfgtlab3[1:6,]
pt <- ggplot(dfgt,aes(x=d,y=w,group=as.character(p)) ) + 
  geom_hline(yintercept=1,color="gray90") +   geom_hline(yintercept=0,color="gray90") +   geom_vline(xintercept=0,color="gray90") + 
  geom_line() +
  geom_text(data=dfgtlab4,aes(x=d,y=.45,label=p), size=2., nudge_x=c(70,120,90,70,40,50) ) +
  xlab("Distance") + ylab("Distance weight"); pt
saveIt("FigureS1",pt,3,3)
rm(dmax,dthresh,p0,d0,p,d,dfgt,dfgtlab,labmax,dfgtlab2,dfgtlab3,dfgtlab4,pt,genthrsh)
###end  FigureS1

#FigureS2
#plot weighting functions versus distance
fd <- data.frame( d=c(1606, 2405,seq(10,100,10),seq(0,4000,100 )) )
fd$step <- ifelse(fd$d<1606,1,0)
fd$lin <- ifelse(fd$d<2405,1-fd$d/2405,0)
fd$exp <- exp(-0.00103*fd$d)
fd$pow <- (fd$d+1)**(-0.798)
fd$dkm <- fd$d/1000
fd <- fd[order(fd$d),]
pt <- ggplot(fd,aes(x=dkm,y=exp)) + 
  geom_hline(yintercept=0,color="gray90") + 
  geom_vline(xintercept=0,color="gray90") + 
  geom_line(color="red") +
  geom_line(aes(x=dkm,y=step),color="black") +
  geom_line(aes(x=dkm,y=lin),color="blue") +
  geom_line(aes(x=dkm,y=pow),color="magenta") +
  coord_cartesian(xlim=c(0,4)) + 
  xlab("Distance (km)") + ylab("Distance Weight"); pt
saveIt("FigureS2",pt,3,3)
rm(fd,pt)
###end FigureS2

#FigueS4
#plots of effective proportion vs. actual proportion
#plot weighted land proportion vs actual land proportion
# and 1:1 line (gray)
#two subestuaries with pcb>300 highlighted in red
modVec <- c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
#set up a vectors for translating legend labels
numname.labs <- c("Unweighted","Step","Linear","Exponential","Power")
names(numname.labs) <- c("01Unwtd","13StepDbl","15PieceDbl","19ExpFitDbl","22InvFitDbl(d+1)")
subestVec <- c(3,4,29,32)
obsResSel <- subset(obsRes,subset=(name %in% modVec) )
obsResLab <- subset(obsResSel,subset=(id %in% subestVec))
pt <- ggplot(obsResSel,aes(x=prop,y=effprop)) + geom_abline(intercept=0,slope=1,color="gray") + geom_point() +
  facet_wrap(vars(numname),ncol=1,labeller=labeller(numname=numname.labs)) + geom_point(data=obsResLab,color="red") +
  geom_text(data=obsResLab,aes(x=prop,y=effprop,label=id), size=2.5,  nudge_x=c(0,0,0.008,0.01), nudge_y=c(.05,.05,-.03,-.02)  ) +
  xlab("Commercial land proportion") +ylab("Weighted commercial land proportion"); pt
saveIt("FIgureS4",pt,2,6)
rm(modVec,numname.labs,obsResSel,obsResLab,pt)
###end of FigureS4

#FigureS5 
#changes in PCB delivery with distance for 4 models and 5 subestuaries (including aggregate)
modVec <- c("StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
#set up a vectors for translating legend labels
numname.labs <- c("Unweighted","Step","Linear","Exponential","Power")
names(numname.labs) <- c("01Unwtd","13StepDbl","15PieceDbl","19ExpFitDbl","22InvFitDbl(d+1)")
id.labs <- c("Aggregate","3","4","29","32")
names(id.labs) <- c("0","3","4","29","32")
subestVec <- c(0,3,4,29,32)
obsBandSel<-subset(obsBand,subset=(name %in% modVec & id %in% subestVec))
pt <- ggplot(obsBandSel,aes(x=dkm,weight=delivrdDiff)) + coord_cartesian(xlim=c(0,10)) + 
  geom_histogram(binwidth=.1) +
  geom_histogram(data=obsBandSel[obsBandSel$delivrdDiff<0,],binwidth=.1,color="red",fill="red") + 
  geom_hline(yintercept=0,color="gray") +
  facet_grid(rows=vars(id),cols=vars(numname),scale="free",labeller=labeller(id=id.labs,numname=numname.labs)) +
  xlab("Distance (km)") + ylab("Delivered PCB - delivered PCB from unweighted model"); pt
saveIt("FigureS5",pt,6,6)
rm(modVec,numname.labs,id.labs,subestVec,obsBandSel,pt)
###end FigureS5

#FIgureS6
#Another way to visualize where PCB delivery is reduced and where it is increased by double weighting
#plot delivrd on top of plot of delivrd from unweighted
#create and apply function to make variable for overlaid shading
makeLow <- function(df ) {
  if (df$delivrd<df$delivrdUnwtd) {delivrdLow<-df$delivrd;delivrdMed<-df$delivrdUnwtd}  else {delivrdLow<-NA;delivrdMed<-NA};
  return(data.frame(delivrdLow,delivrdMed))
}
modVec <- c("Unwtd","StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
#set up a vectors for translating legend labels
numname.labs <- c("Unweighted","Step","Linear","Exponential","Power")
names(numname.labs) <- c("01Unwtd","13StepDbl","15PieceDbl","19ExpFitDbl","22InvFitDbl(d+1)")
subestVec <- c(0,3,4,29,32)
id.labs <- c("Aggregate","3","4","29","32")
names(id.labs) <- c("0","3","4","29","32")
obsBandSel<-subset(obsBand,subset=(name %in% modVec & id %in% subestVec))
obsBandSel2 <- adply(obsBandSel, 1, makeLow,.progress="text")  ##includes unwtd
#delivered by model and subestuary
#with amounts above Unwtd shaded blue and amounts below Unwtd shaded red
pt <- ggplot(obsBandSel2,aes(x=dkm,weight=delivrd)) + coord_cartesian(xlim=c(0,10)) + 
  geom_histogram(binwidth=.1,color="dodgerblue",fill="dodgerblue") +
  geom_histogram(aes(x=dkm,weight=delivrdUnwtd),binwidth=.1,color="gray",fill="gray") +
  geom_histogram(aes(x=dkm,weight=delivrdMed),binwidth=.1,color="salmon",fill="salmon") +
  geom_histogram(aes(x=dkm,weight=delivrdLow),binwidth=.1,color="gray",fill="gray") +
  facet_grid(rows=vars(id),cols=vars(numname),scale="free",labeller=labeller(id=id.labs,numname=numname.labs)) +
  xlab("Distance (km)") + ylab("Delivered PCB"); pt
saveIt("FigureS6",pt,6,6)
rm(modVec,numname.labs,id.labs,subestVec,obsBandSel,obsBandSel2,pt,makeLow)
###end FigureS6

#FigureS7
#plot realized weighting functions
myfun <- function(dtfrm) {
  df <- dtfrm
  df$wgt <- NA
  if (df$name[1]=="StepDbl") {
    t <- results[results$name=="StepDbl","t"]
    df$wgt <- ifelse(df$d<=t,1,0)
  }
  if (df$name[1]=="PieceDbl") {
    t <- results[results$name=="PieceDbl","t"]
    df$wgt <- ifelse(df$d<=t,1-df$d/t,0)
  }
  if (df$name[1]=="ExpFitDbl") df$wgt <- exp(-df$d*results[results$name=="ExpFitDbl","k"])
  if (df$name[1]=="InvFitDbl(d+1)") df$wgt <- df$d**-results[results$name=="InvFitDbl(d+1)","p"]
  return(df)
}
modVec <- c("StepDbl","PieceDbl","ExpFitDbl","InvFitDbl(d+1)")
numname.labs <- c("Unweighted","Step","Linear","Exponential","Power")
names(numname.labs) <- c("01Unwtd","13StepDbl","15PieceDbl","19ExpFitDbl","22InvFitDbl(d+1)")
subestVec <- c(0,3,4,29,32)
id.labs <- c("Aggregate","3","4","29","32")
names(id.labs) <- c("0","3","4","29","32")
obsBandSel<-subset(obsBand,subset=(name %in% modVec & id %in% subestVec))
obsBandSel$yb <- obsBandSel$propWtd/obsBandSel$prop
obsBandSel2 <- ddply(obsBandSel,.(name),myfun)
obsBandSel2$weightRatio <- round(obsBandSel2$yb/obsBandSel2$wgt,digits=4)
pt <- ggplot(obsBandSel2,aes(x=dkm,y=yb))  + coord_cartesian(xlim=c(0,5)) +
  geom_line(aes(x=dkm,y=wgt),color="red") + geom_line() +
  facet_grid(cols=vars(id),rows=vars(numname),scale="free",labeller=labeller(id=id.labs,numname=numname.labs)) + 
  ylab("Realized Distance Weight") + xlab("Distance (km)"); pt
saveIt("FIgureS7",pt,6,6)
rm(modVec,numname.labs,id.labs,subestVec,obsBandSel,obsBandSel2,pt,myfun)
###end FigureS7

# FigureS8
#distance distributions of all land and commercial land for aggregate and for 4 selected subestuaries
#kludge needed because lines at y=0 extend beyond domain of data, fix by overlaying white line from upper limit
#of domain to the rignt end of the axis, this hides the unwanted line where there is no data
#also use for adding subestuary number to the plot panesl
subestVec <- c(0,3,4,29,32)  #aggregate + 4 individual watersheds
kk <- data.frame(id=subestVec, ndmax=c(45.85,10.82,13.06,8.96,29.31), ddmax=c(47.18,10.89,13.18,9.12,29.47), maxmax=rep(47.18,5),
                 xv=rep(45,5), yv=c(140,2.7,7.5,5.9,7.5),  idtxt=c("Aggregate","3","4","29","32"), stringsAsFactors=F )
obsBandSel<-subset(obsBand,subset=(modnum==1 & id %in% subestVec))
obsBandSel <- merge(obsBandSel,kk,by="id")
pt <- ggplot(data=obsBandSel, aes(x=dkm,weight=darea/100)) + 
  geom_histogram(binwidth=.2,color="gray80",fill="gray80") +
  geom_histogram(aes(x=dkm,weight=narea/100),color="black",fill="black",binwidth=.2) +
  geom_segment(aes(x=ndmax+.1,y=0,xend=maxmax,yend=0),color="gray80") +
  geom_segment(aes(x=ddmax+.1,y=0,xend=maxmax+.1,yend=0),color="white") +
  facet_grid(rows=vars(id),scale="free") +
  geom_text(aes(x=xv,y=yv,label=idtxt),size=2.5, hjust=1, fontface="plain") +
  theme(strip.background = element_blank(), strip.text.x = element_blank(), strip.text.y = element_blank()) +
  ylab(expression("Area km"^2)) + xlab("Distance km"); pt
saveIt("FigureS8",pt,3,6)
rm(kk,obsBandSel,pt)
#bin width of .2 (200 meters) with 20 km max gives 100 bins
#used the following to figure out where to draw white lines in above kludge
#qq <- obsBandSel[obsBandSel$modnum==1 & obsBandSel$id==0,]; max(qq[qq$narea>0,"d"]); max(qq$d)
###end of FigureS8
