#5FiguresS3.R


#Code to plot FigureS3 from this published paper:
#New methods for quantifying the effects of catchment spatial patterns on aquatic responses
#by Donald E. Weller, Matthew E. Baker, and Ryan S. King
#Landscape Ecology https://doi.org/10.1007/s10980-023-01706-x

#Don Weller, wellerd@si.edu, Smithsonian Environmental Research Center, Edgewater, MD  21037
#last tested with Windows 10 RStudio 2023.06.0+421 "Mountain Hydrangea"
#and R version 4.3.0 (2023-04-21 ucrt) "Already Tomorrow"
#June 2023

#construct plots to show how AICc varies with weighing parameters for four weighing functions
#step, linear, exponential, and power
#with double weighting, each function gives a clear global minimum at a single, optimum parameter value
#with single weighting, AICc never falls below the value for the unweighted model and there is no
#globally optimal parameter value

library(plyr)
library(ggplot2)
library(grid)
my_theme <- theme_bw(base_size=9) + theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
  theme(axis.text= element_text(color="black")) +
  theme(axis.title.x = element_text(size = 9), axis.text.x = element_text(size = 8),
        axis.title.y = element_text(size = 9), axis.text.y = element_text(size = 8)  )
theme_set(my_theme)

#get functions from other R files
source("DistWeightFunctions.R")
source("AuxillaryFunctions.R")

#figure will be saved as a .tif file to the subdirectory called FiguresTables
#if the directory does not exist yet, create it
if (!dir.exists("FiguresTables"))  dir.create("FiguresTables")

#function to fit one model & return fit statistics given one parameter value, a weighting function, and a double-weighting flag
#because the parameter value is specified (rather than fit) the model is a simple linear regression, so use lm for fitting
doOne <- function(pval,wEqn,doublFlag) {
  param <- c(0,1); names(param) <- c("b0","b1")
  numWeqn <- wEqn(pval) 
  denWeqn <- ifelse (doublFlag,numWeqn,"1")
  if (doublFlag) denWeqn <- numWeqn else denWeqn<-"1"
  #calculate the weighted proportions from the distance distributions and weighting function info
  wtdprop <- myPred( param,numWeqn,denWeqn,ad4$id,ad4$d,ad4$cc,ad4$tc,FALSE )
  pcb2 <- cbind(pcb,wtdprop)                #combine dependent and indepentent vars
  mod <- lm(pcb~wtdprop,pcb2)               #fit the linear model
  RSS <- sum( summary(mod)$residuals**2 )   #calculate residual sum of squares
  R2 <- summary(mod)$r.squared              #then R2
  #note: calculate AICc as if the weighting parameter HAD been fit (i.e., use k=4)
  AICc <- myAICcSSR(RSS,14,4)
  data.frame(pval,RSS,R2,AICc)
}

#step and piecewise weighting functions parameter domain
pvals <- c( seq(60,3000,10), seq(3100,10000,100), seq(11000,50000,1000))
#step function
#function to return the step weighting  equation as text (for a single pval)
wEqn <- function(pval) paste0( "sapply(d,function(x) {ifelse(x<=", pval, ",1,0)} )"  )
stpSgl <- adply (pvals, 1, doOne, wEqn, F)
stpDbl <- adply (pvals, 1, doOne, wEqn, T)
#linear function
wEqn <- function(pval) paste0( "sapply(d,function(x) {ifelse(x<=", pval, ",1-x/", pval, ",0)} )"  )
linSgl <- adply (pvals, 1, doOne, wEqn, F)
linDbl <- adply (pvals, 1, doOne, wEqn, T)
#exponential function
log10p <- seq(-6,-2,.05)
pvals <- 10**(log10p)      #exponential parameter domain
wEqn <- function(pval) paste0( "exp(-", pval, "*d)"  )
expSgl <- adply (pvals, 1, doOne, wEqn, F)
expDbl <- adply (pvals, 1, doOne, wEqn, T)
#power function
pvals <- seq(0,3,.05)      #power function parameter domain
wEqn <- function(pval) paste0( "(d+1)**(-", pval, ")"  )
powSgl <- adply (pvals, 1, doOne, wEqn, F)
powDbl <- adply (pvals, 1, doOne, wEqn, T)

###now make the plots
#each plot shows the parameter optimization curve for single weighting in red and double weighting in blue
yint=185.376936103765  #AICc for ssr of unwtd model with one more parameter

#identify the local minimum in AICc for low values of the parameter
earlyMin <- subset(stpSgl[1:300,],subset=(AICc==min(AICc)))
index <- as.numeric(row.names(earlyMin))
stpp <- ggplot(stpSgl,aes(pval,AICc)) + geom_abline(slope=0,intercept=yint, color="gray") +
  geom_line(color="red")+ geom_line(data=stpDbl,aes(pval,AICc),color="blue") +
  geom_point( data=stpSgl[index,],aes(x=pval,y=AICc),color="red") +
  coord_cartesian( xlim = c(0,20000),ylim = c(120,200) ) +
  labs(x=expression( italic(t)[s]), y="AICc",title="Step");  stpp

#identify the local minimum in AICc for low values of the parameter
earlyMin <- subset(linSgl[1:300,],subset=(AICc==min(AICc)))
index <- as.numeric(row.names(earlyMin))
linp <- ggplot(linSgl,aes(pval,AICc)) + geom_abline(slope=0,intercept=yint, color="gray") +
  geom_line(color="red")+ geom_line(data=linDbl,aes(pval,AICc),color="blue") +
  geom_point( data=linSgl[index,],aes(x=pval,y=AICc),color="red") +  coord_cartesian( xlim = c(0,20000),ylim = c(120,200) ) +
  labs(x=expression( italic(t)[l]), y="AICc",title="Linear") +
  theme(axis.title.x = element_text(face="italic") ); linp

#identify the local minimum in AICc for low values of the parameter
earlyMin <- subset(expSgl[expSgl$pval>0.00003,],subset=(AICc==min(AICc)))
index <- as.numeric(row.names(earlyMin))
expp <- ggplot(expSgl,aes(pval,AICc)) + geom_abline(slope=0,intercept=yint, color="gray") +
  geom_line(color="red")+ geom_line(data=expDbl,aes(pval,AICc),color="blue") +
  geom_point( data=expSgl[index,],aes(x=pval,y=AICc),color="red") +
  coord_cartesian( xlim = c(0,.01),ylim = c(120,200) ) +
  labs(x="k",y="AICc",title="Exponential") +
  theme(axis.title.x = element_text(face="italic") ); expp

powp <- ggplot(powSgl,aes(pval,AICc)) + geom_abline(slope=0,intercept=yint, color="gray") +
  geom_line(color="red")+ geom_line(data=powDbl,aes(pval,AICc),color="blue") +
  coord_cartesian( xlim = c(0,3),ylim = c(120,200) ) +
  labs(x="p",y="AICc",title="Power") +
  theme(axis.title.x = element_text(face="italic") ); powp

#now put the 4 plots in a single page using grid viewports
vp1 <- viewport(height=.25,width=1,y=0,x=0,just=c("left","bottom"))
vp2 <- viewport(height=.25,width=1,y=.25,x=0,just=c("left","bottom"))
vp3 <- viewport(height=.25,width=1,y=.50,x=0,just=c("left","bottom"))
vp4 <- viewport(height=.25,width=1,y=.75,x=0,just=c("left","bottom"))
#
#save the finished plot as a tiff file
tiff(".\\FiguresTables\\FigureS3.tif", width=3,height=6,units="in",res=600,compression='lzw',restoreConsole=T);
print(stpp,vp=vp4); print(linp,vp=vp3); print(expp,vp=vp2); print(powp,vp=vp1);
dev.off()

#clean up
rm(doOne,wEqn,pvals,log10p,stpSgl,stpDbl,linSgl,linDbl,expSgl,expDbl,powSgl,powDbl,stpp,linp,expp,powp)
rm(yint,earlyMin,index,vp1,vp2,vp3,vp4)
