#AuxillaryFunctions.R

#R functions used in the analyzis in this published paper:
#New methos for quantifying the effects of catchment spatatial patterns on aquatic responses
#by Donald E. Weller, Matthew E. Baker, and Ryan S. King
#Landscape Ecology xx:xx-xx 2023
#DOI https://xxxx

#Don Weller, wellerd@si.edu, Smithsonian Environmental Research Center, Edgewater, MD  21037
#last tested with Windows 10 RStudio 2023.06.0+421 "Mountain Hydrangea"
#and R version 4.3.0 (2023-04-21 ucrt) "Already Tomorrow"
#June 2023

#function to calculate AICc for a least squares model from observed, predicted,
#and number of parameters K (including the intercept and sigma**2)
myAICc <- function(observed,predicted,K) {
  resid <- observed-predicted
  RSS <- sum(resid**2)
  N<-length(resid)
  #formula includes the constant log(2*pi)+1 from the liklihood for consistency with other R AIC calculations
  #Burnham and Anderson omit it, which does NOT affect delta calculations, but is not consistent with other R functions
  AICc <- N*( log(RSS/N) + log(2*pi) + 1 ) + 2*K + 2*K*(K+1)/(N-K-1)
  return(AICc)
}
#end of function myAICc

#function to calculate AICc for a least squares model from the residual sum of squares, the sample size,
#and number of parameters K (including the intercept and sigma**2)
myAICcSSR <- function(RSS,N,K) {
  #formula includes the constant log(2*pi)+1 from the liklihood for consistency with other R AIC calculations
  #Burnham and Anderson omit it, which does NOT affect delta calculations, but is not consistent with other R functions
  AICc <- N*( log(RSS/N) + log(2*pi) + 1 ) + 2*K + 2*K*(K+1)/(N-K-1)
  return(AICc)
}
#end of function myAICcSSR

#function to calculate R2 from observed and predicted
#note: observed must be sorted in the same order accoring to observation numbers
myR2 <- function (observed,predicted) {
  #R2 is squared correlation of predicted with observed
  R2 <- cor(predicted,observed)**2
  return(R2)
}
#end of function myR2

#function to calculate R2 from observed and sum of squared residuals
myR2SSR <- function (observed,SSR) {
  TSS <- sum( (observed-mean(observed))**2  )
  R2 <- 1 - SSR/TSS
  return(R2)
}
#end of function myR2SSR

#function to create a coefficient matrix from an Optimr object and the number of observations
#returns a matrix with parameters, se, t, and P values--like  coef.lm output
coefOptimr<- function(modObj,nobs) {
  par <- unlist(modObj$par)             #convert parameters from list to named vector
  npar <- length(modObj$par)            #number of parameters
  H <- modObj$hessian                   #Hessian matrix
  #the follow will fail for some models. If so, try function will return an object of class "try-error"
  #and allow continued processing
  invH2 <- try(solve((H/2)),silent=T)     #invert half hessiant
  #set SE, T, and P missing if couldn't invert Hessian
#######  if (class(invH2)=="try-error") {SE <- rep(NA,npar); T<-SE; P<-SE} else {
  if (inherits(invH2, "try-error")) {SE <- rep(NA,npar); T<-SE; P<-SE} else {
     #otherwise, proceed to calculate SE, T, and P
     ssr <- modObj$value
     #fitted ssr (mySSR at final parameter values)
     edf <- nobs-npar                       #residual (error) degrees of freedom
     sigma2 <- ssr/edf                     #error mean square = ssr / edf
     #covariance matrix is inverse Hessian/2 scaled by sigma2          
     Cov <- sigma2*invH2
     Var <- diag(Cov)                      #variances are on diagonal of covariance matrix
     #numerical issues in optimization can sometimes result in negative variances here
     #don't attempt sqrt if negative
     SE <- sapply( Var, function (x) ifelse(x<0,NA,sqrt(x)) )         #standard errors
     t <- par/SE                           #t values
     P <- 2 * pt(abs(t), edf, lower.tail = F)     #P values
     }
  #now assemble into matrix and return
  coef <- cbind(par,SE,T,P)
  return(coef)
}
#end of function coefOptimr



#function to create a coefficient matrix from parameter list,  SSR, Hessian matrix, and number of observatons
#returns a coefficient matrix with parameters, se, t, and P values
coefFromHess<- function(param,ssr,H,nobs) {
  npar  <-length(param)
  #now use the Hessian to calculate standard errors for the parameters  
  #the following will fail for some Hessians. If so, try function will return an object of class "try-error"
  #and allow continued processing
  invH2 <- try(solve((H/2)),silent=T)     #invert half hessian
  #set SE, T, and P missing if couldn't invert Hessian
#######  if (class(invH2)=="try-error" ) {SE <- rep(NA,npar); T<-SE; P<-SE} else {
  if (inherits(invH2, "try-error")) {SE <- rep(NA,npar); T<-SE; P<-SE} else {
    #otherwise, proceed to calculate SE, T, and P
    edf <- nobs-npar                       #residual (error) degrees of freedom
    sigma2 <- ssr/edf                     #error mean square = ssr / edf
    #covariance matrix is inverse Hessian/2 scaled by sigma2          
    Cov <- sigma2*invH2
    Var <- diag(Cov)                      #variances are on diagonal of covariance matrix
    #numerical issues or poor solutions can sometimes result in negative variances here
    #don't attempt sqrt if negative, assign NA instead
    SE <- sapply( Var, function (x) ifelse(x<0,NA,sqrt(x)) )         #standard errors
    T <- param/SE                           #t values
    P <- 2 * pt(abs(T), edf, lower.tail = F)     #P values
  }
  #now assemble into matrix and return
  coef <- cbind(param,SE,T,P)
  return(coef)
}
#end of function coefFromHess


#function to fetch a data frame from aonther workspace
dfFetch <- function(dfname,workdir) {
  tmp.env <- new.env() # create a temporary environment
  load(paste0(workdir,".RData"), envir=tmp.env)
  dframe <- get(dfname, pos=tmp.env) # get the objects you need into your globalenv()
  rm(tmp.env) # remove the temporary environment to free up memory
  return(dframe)
}
#end of function dfFetch

