#DistWeightFunctions.R

#R functions used in the analysis in this published paper:
#New methods for quantifying the effects of catchment spatial patterns on aquatic responses
#by Donald E. Weller, Matthew E. Baker, and Ryan S. King
#Landscape Ecology xx:xx-xx 2023
#DOI https://xxxx

#Don Weller, wellerd@si.edu, Smithsonian Environmental Research Center, Edgewater, MD  21037
#last tested with Windows 10 RStudio 2023.06.0+421 "Mountain Hydrangea"
#and R version 4.3.0 (2023-04-21 ucrt) "Already Tomorrow"
#June 2023

#function to calculate a residuals vector for a parameter set
myResid <- function(param,predfun,numWeqn,denWeqn,observed,id,dist,narea,darea,allFlag)
{  observed-predfun(param,numWeqn,denWeqn,id,dist,narea,darea,allFlag)  }
#end of function myResid

#function to calculate the sum of squared residuals for a paramter set
mySSR <- function(param,predfun,numWeqn,denWeqn,observed,id,dist,narea,darea,allFlag)
{  sum( (observed-predfun(param,numWeqn,denWeqn,id,dist,narea,darea,allFlag))**2 )  }
#end of function mySSR

#function to calculate predicted values for a parameter set & weighting functions
#applies weighting functions across the distance bands
myPred <- function(param,numWeqn,denWeqn,id,d,narea,darea,allFlag) {
  #if param is a list, convert it to a named vector
  if (class(param)=="list") parnum<-unlist(param) else parnum<-param
  #create and execute command to define individual parameters from the named list
  pardef <- paste0(names(parnum),"<-",parnum)  #creates vector of R assignment commands
  eval(parse(text=pardef))               #executes the commands to set parameter values
  #the following will execute a statement like: numWgt<-exp(-d*p)
  #where the RHS will be the expression in numWeqn, which is the numerator weighting equation as text
  numWgt <- eval(parse(text=numWeqn))
  denWgt <- eval(parse(text=denWeqn))   #same for denominator weighting equation
  #following yield vectors of weighted numerator and denominator areas by subestuary and distance
  nwtd <- narea*numWgt  #apply numerator distance weight to source area
  dwtd <- darea*denWgt  #apply denominator distance weight to total area
  #now sum across distance bands to get totals per subestuary, but
  #if allFlag==T, tapply NOT used so sum is accross entire data set (all observations)
  if (allFlag==T) {numer<-sum(nwtd);denom<-sum(dwtd)} else 
    {numer <- tapply(nwtd,id,sum); denom <- tapply(dwtd,id,sum) }
  #implement the weighted linear regression equation
  #note:  this equation works on land proportions as fractions, not percents
  #(numer/denom) is the "effective land proportion"
  #param MUST include b0 and b1
  pred <- b0 + b1*(numer/denom)
  return(pred)
}
#end of function myPred

#function to calculate the bandWeight values for a parameter set & weighting functions
#applies weighting functions across all the distance bands in all watersheds
#returns band data and weighting results in a data frame
bandWeights <- function(param,numWeqn,denWeqn,id,d,narea,darea) {
  #if param is a list, convert it to a named vector
  if (class(param)=="list") parnum<-unlist(param) else parnum<-param
  #create and execute command to define individual parameters from the named list
  pardef <- paste0(names(parnum),"<-",parnum)  #creates vector of R assignment commands
  eval(parse(text=pardef))               #executes the commands to set parameter values
  eval(parse(text=pardef))               #executes the commands to set parameter values
  #the following will execute a statement like: numWgt<-exp(-d*p)
  #where the RHS will be the expression in numWexp for the current model
  numWgt <- eval(parse(text=numWeqn))
  denWgt <- eval(parse(text=denWeqn))
  nwtd <- narea*numWgt  #apply numerator distance weight to source area
  dwtd <- darea*denWgt  #apply denominator distance weight to total area
  weightdf <- data.frame(cbind(id,d,narea,darea,numWgt,denWgt,nwtd,dwtd))
  return(weightdf)
}
#end of function bandWeights

#function to test a lot of initial parameter sets and pick the best starting parameter set
#assumes parameters come in a named lists, with each list element being a vector of possible
#starting values for one parameter
bestFromMulti <- function( param, resfun, predfun, numWeqn, denWeqn, observed, id, dist, narea, darea ) {
  parlengths <- lengths(param)   #vector of the numbers of starting values in each list element
  npars <- length(parlengths)    #number of parameters
  cp <- cumprod(parlengths)      #cumulative product of the numbers of starting values
  nsets <- cp[length(cp)]        #last cumulative product is total number of parameter sets
  #if only one parameterset offered, just return it
  if (nsets==1) { bestpar<-unlist(param) }   
  else   {
    #if more than one set, then estimate ssr for each one
    #the following gives the number of times each parameter must be repeated sequentially
    #when filling out the matrix of all possible parameter sets
    nreps <- c(1,cp[1:(npars-1)])  #value for "each" parameter in following rep function in mapply
    #the following creates all possible combinations of the parameters in a matrix (rows=sets, cols= the parameters)
    parmat <- mapply( function(x,y) {rep(x,each=y,length.out=nsets)}, x=param, y=nreps )
    #now create a vector of sums of square residuals for all the parameter set
    #use aaply from plyr instead of apply, because plyr version includes a progress par
    ssr <- aaply( parmat,1,
         function(x) {mySSR( x, predfun, numWeqn, denWeqn, observed, id, dist, narea, darea, allFlag=F) }, .progress="text" )
    minssr <- min(ssr)
    both <- cbind(parmat,ssr)             #combine ssrs with params in one matrix
    bestparssr <- both[ssr==minssr,]      #subset of matrix with the minimum ssr (hopefully just one)
    #if more than one set have the same ssr (matrix of minimums), just choose the first row
    if (class(bestparssr)=="matrix") { bestpar<-bestparssr[1,1:npars] } else { bestpar<-bestparssr[1:npars] } 
    }
  return(bestpar)
}
#end of function bestFromMulti